from typing import TYPE_CHECKING

from based_utils.calx import fractions
from based_utils.cli import Colored
from based_utils.colors import COLORS, HUES, Color, Hues

if TYPE_CHECKING:
    from collections.abc import Iterator


def _color_shades(c: Color) -> Iterator[str]:
    for f in fractions(19):
        s = c.shade(f)
        yield Colored(f" {s.hex} ", s.contrasting_shade, s).formatted


def _color_line(c: Color, n: str = "") -> str:
    h = "   " if n == "grey" else f"{c.hue * 360:03.0f}"
    return "".join(_color_shades(c)) + Colored(f" {h} {n}", c).formatted


def _saturation_lines(saturation: float, color_cls: type[Color]) -> Iterator[str]:
    if saturation == 0:
        yield _color_line(color_cls.grey(), "grey")
    else:
        yield "".join(f"   {f * 100:.0f}%  " for f in fractions(19))
        for h in COLORS:
            yield _color_line(color_cls.from_name(h, saturation=saturation), h)


def color_lines(custom_hues: Hues) -> Iterator[str]:
    class CustomColor(Color):
        hues = HUES | custom_hues

    for saturation in fractions(3, inclusive=True):
        yield from _saturation_lines(saturation, CustomColor)



import argparse

from based_utils.cli import LogLevel, check_integer_within_range
from based_utils.colors import HUES, Color

from . import log
from .color_theme import color_lines
from .css_variables import InterpolationParams, shades_as_css_variables

_log = log.get_logger()


def css_variables(args: argparse.Namespace) -> None:
    with log.context(LogLevel.INFO):
        for line in shades_as_css_variables(
            Color.from_hex(args.color1),
            Color.from_hex(args.color2),
            params=InterpolationParams(
                args.amount, args.inclusive, args.dynamic_range / 100
            ),
            label=args.label,
        ):
            _log.info(line)


def color_theme(args: argparse.Namespace) -> None:
    with log.context(LogLevel.INFO):
        for line in color_lines(
            {h: v for h in HUES if (v := getattr(args, h)) is not None}
        ):
            _log.info(line)


def main() -> None:
    parser = argparse.ArgumentParser()
    subparsers = parser.add_subparsers(required=True)

    parser_css_variables = subparsers.add_parser("css")
    parser_css_variables.add_argument("label", type=str)
    parser_css_variables.add_argument("-c", "--color1", type=str)
    parser_css_variables.add_argument("-k", "--color2", type=str, default=None)
    parser_css_variables.add_argument(
        "-n", "--amount", type=check_integer_within_range(0, None), default=19
    )
    parser_css_variables.add_argument(
        "-i", "--inclusive", action="store_true", default=False
    )
    parser_css_variables.add_argument(
        "-d", "--dynamic-range", type=check_integer_within_range(0, 100), default=0
    )
    parser_css_variables.set_defaults(func=css_variables)

    parser_color_theme = subparsers.add_parser("colors")
    for color in HUES:
        parser_color_theme.add_argument(f"--{color}", dest=color, type=int)
    parser_color_theme.set_defaults(func=color_theme)

    args = parser.parse_args()
    args.func(args)



from dataclasses import dataclass
from typing import TYPE_CHECKING

from based_utils.calx import (
    CyclicInterpolationBounds,
    InterpolationBounds,
    fractions,
    interpolate,
    trim,
)
from based_utils.cli import Colored
from based_utils.colors import Color

from . import log

if TYPE_CHECKING:
    from collections.abc import Iterator


_log = log.get_logger()


@dataclass(frozen=True)
class InterpolationParams:
    n: int = 19
    inclusive: bool = False
    dynamic_range: float = 0


def _shades_2(
    color_1: Color, color_2: Color, *, params: InterpolationParams
) -> Iterator[Color]:
    c_dark, c_bright = sorted([color_1, color_2])

    dynamic_range = params.dynamic_range
    l_dark = interpolate(dynamic_range, start=c_dark.lightness, end=0)
    l_bright = interpolate(dynamic_range, start=c_bright.lightness, end=1)
    lightness_bounds = InterpolationBounds(l_dark, l_bright)

    saturation_bounds = InterpolationBounds(c_dark.saturation, c_bright.saturation)
    hue_bounds = CyclicInterpolationBounds(c_dark.hue, c_bright.hue)

    for lightness in fractions(params.n, inclusive=params.inclusive):
        f = lightness_bounds.inverse_interpolate(lightness, inside=False)
        hue = hue_bounds.interpolate(f)
        saturation = trim(saturation_bounds.interpolate(f))
        yield Color(lightness, saturation, hue)


def _generate_shades(
    color_1: Color, color_2: Color = None, *, params: InterpolationParams
) -> Iterator[Color]:
    return (
        _shades_2(color_1, color_2, params=params)
        if color_2
        else color_1.shades(params.n, inclusive=params.inclusive)
    )


def _colored(s: str, color: Color) -> str:
    return Colored(s, color.contrasting_shade, color).formatted


def _css_color_comment(color: Color) -> str:
    return _colored(f"#{color.hex} --> {color}", color)


def shades_as_css_variables(
    c_1: Color, c_2: Color | None, *, params: InterpolationParams, label: str
) -> Iterator[str]:
    yield "/*"
    yield "Based on:"
    if c_2:
        c_dark, c_bright = sorted([c_1, c_2])
        yield f"- Darkest:   {_css_color_comment(c_dark)}"
        yield f"- Brightest: {_css_color_comment(c_bright)}"
    else:
        yield _css_color_comment(c_1)
    yield "*/"

    for color in _generate_shades(c_1, c_2, params=params):
        num = int(color.lightness * 100)
        color_var = f"--{label}-{num:02d}: #{color.hex}; /* --> {color} */"
        yield _colored(color_var, color)
